#include "LLtest.hh"

using namespace std;

/********* Constructor and Destructor ************/
LRtest::LRtest( scanData * s ) {

  sd = s;
  LLR = new double*[sd->numSizes( )];
  for ( int i = 0 ; i < sd->numSizes( ) ; i++ ) {
    LLR[i] = new double[2*sd->numVariants()];
  }

  LLRmax = new double[sd->numSizes()+1];
  LLRmax_size = new int[sd->numSizes()+1];
  LLRmax_window = new int[sd->numSizes()+1];

}

LRtest::~LRtest( ) {

  for ( int i = 0 ; i < sd->numSizes( ) ; i++ ) {
    delete[] LLR[i];
  }
  delete[] LLR;

}

/************ LLR Methods ***************/
void LRtest::calc_LLR( int size, int win ) {

  // Get window bounds
  int win_lower = sd->WinStart( size, win );
  int win_upper = sd->WinEnd( size, win );

  // Initialize parameters
  double yw = 0;
  double yg = 0;
  double nw = 0;
  double ng = 0;
  double pw;
  double qw;
  double rg;

  // Calculations
  for ( int i = 0 ; i < win_lower ; i++ ) {
    yg += sd->yu( sd->map(i) );
    ng += sd->nu( sd->map(i) );
  }
  for ( int i = win_lower ; i <= win_upper ; i++ ) {
    yg += sd->yu( sd->map(i) );
    ng += sd->nu( sd->map(i) );
    yw += sd->yu( sd->map(i) );
    nw += sd->nu( sd->map(i) );
  }
  for ( int i = (win_upper+1) ; i < sd->numVarRet( ) ; i++ ) {
    yg += sd->yu( sd->map(i) );
    ng += sd->nu( sd->map(i) );
  }

  pw = (yw+1) / (nw+2);
  qw = (yg-yw+1)/(ng-nw+2);
  rg = (yg+1)/(ng+2);
  if ( pw > qw ) {
    LLR[size][win] = log(pw/rg) * yw + log((1-pw)/(1-rg)) * (nw-yw) + log(qw/rg) * (yg-yw) + log((1-qw)/(1-rg)) * (ng-nw-yg+yw);
  } else {
    LLR[size][win] = 0;
  }
  //cout << "LLtest: " << pw << "\t" << qw << "\t" << rg << "\n";
}

void LRtest::calc_LLR_all( ) {

  int win_lower;
  int win_upper;

  LLRmax[sd->numSizes()] = 0;
  LLRmax_window[sd->numSizes()] = 0;
  LLRmax[sd->numSizes()] = 0;
  for ( int j = 0 ; j < sd->numSizes( ) ; j++ ) {
    LLRmax_size[j] = 0;
    LLRmax_window[j] = 0;
    LLRmax[j] = 0;
    for ( int i = 0 ; i < sd->numWindows(j) ; i++ ) {
      win_lower = sd->WinStart( j, i );
      win_upper = sd->WinEnd( j, i );
      calc_LLR( j, i );
      if ( LLR[j][i] > LLRmax[j] ) {
	LLRmax[j] = LLR[j][i];
	LLRmax_size[j] = win_lower;
	LLRmax_window[j] = win_upper;
      }
    }
    if ( LLRmax[j] > LLRmax[sd->numSizes()] ) {
      LLRmax[sd->numSizes()] = LLRmax[j];
      LLRmax_size[sd->numSizes()] = LLRmax_size[j];
      LLRmax_window[sd->numSizes()] = LLRmax_window[j];
    }
  }
}

double LRtest::get_LLR( int j, int i ) {
  return LLR[j][i];
}

double LRtest::get_LLRmax( int i ) {
  return LLRmax[i];
}

int LRtest::get_LLRmax_size( int i ) {
  return LLRmax_size[i];
}

int LRtest::get_LLRmax_window( int i ) {
  return LLRmax_window[i];
}

