
#include "ScanData.hh"
#include <vector>
#define PHEN_CODE 2
#define MAX_MAF maf_max

using namespace std;

/************* Constructor **************/
scanData::scanData( const char * ped, const char * var, const char * inc, int * sizes, int num, double max_maf, int chromosome, int l_bound, int u_bound ) {

  /************* Read in data from storage and set data variables ********/
  // set maximum MAF
  maf_max = max_maf;

  // Initialize subject and variant counts to zero
  NumSubjects = 1;
  NumVariants = 0;
  NumCases = 0;
  NumControl = 0;

  // Setup file connections and input processing objects
  ifstream input, input2;
  string line; // Hold lines as they are read in from file
  string line2;
  string entry; // Hold entries from line as they are read by stringstream
  stringstream s, s2; // Used to process lines
  vector<int> in_range;
  vector<int> filter;
  int index, counter, iter;
  int max_index;
  int temp_int, temp_int2;
  int chrom;

  // Read in data from var file
  input.open( var );
  input2.open( inc );
  getline( input, line ); // Discard header line
  index = 0;
  while ( getline( input, line ) ) {
    getline( input2, line2 );
    s.clear( );
    s.str("");
    s2.clear( );
    s2.str("");
    s << line;
    s >> temp_int;
    s2 << line2;
    s2 >> temp_int2;
    chrom = temp_int;
    if ( chrom == chromosome ) {
      s >> temp_int;
      if ( (temp_int >= l_bound) & (temp_int <= u_bound) ) {
	in_range.push_back(index);
	filter.push_back( temp_int2 );
	NumVariants++;
      }
    }
    index++;
  }
  input.close( );
  input2.close( );
  max_index = index;

  // Read in data from inc file
  counter = 0;
  iter = 0;
  /*while ( (getline( input, line )) && (iter < in_range.size()) ) {
    s.clear( );
    s.str("");
    s << line;
    s >> temp_int;
    if ( counter == in_range[iter] ) {
      filter.push_back( temp_int );
      iter++;
    }
    counter++;
  }
  input.close();*/


  // Count rows and columns from data file
  input.open( ped );
  //getline( input, line );
  s.clear( );
  while ( getline( input, line ) ) {
    NumSubjects++;
  }
  input.close( );

  // Set array lengths
  coordinate = new int[NumVariants];
  phenotype = new int[NumSubjects];
  genotype = new int*[NumSubjects];
  for ( int i = 0 ; i < NumSubjects ; i++ ) {
    genotype[i] = new int[NumVariants];
  }
  // Read in data from ped file
  //int temp_int; // Hold integer valued variables from stringstream
  input.open( ped );
  int index2 = 0;
  for ( int i = 0 ; i < NumSubjects ; i++ ) {
    //cout << "Subject " << i << "\n";
    counter = 0;
    index = 0;
    index2 = 0;
    getline( input, line );
    s.clear( );
    s.str("");
    s << line;

    for ( int j = 0 ; j < 5 ; j++ ) { // Discard first 5 entries
      s >> entry;
    }
    s >> temp_int; // 6th entry contains phenotype
    if ( temp_int == PHEN_CODE ) {
      phenotype[i] = 1;
      NumCases++;
    } else {
      phenotype[i] = 0;
      NumControl++;
    }

    for ( int j = 0 ; j < max_index ; j++ ) {
      s >> temp_int;
      if ( (j == in_range[index]) ) {
	if ( filter[counter] == 1 ) {
	  genotype[i][index2] = temp_int;
	  index2++;
	}
	index++;
	counter++;
      }
    }
  }
  input.close( );
  //cout << "Mark5\n";
  // Read in data from var file 
  input.open( var );
  index = 0;
  index2 = 0;
  for ( int i = 0 ; i < max_index ; i++ ) {
    getline( input, line );
    s.clear( );
    s.str("");
    s << line;
    s >> entry; // Discard first entry
    s >> temp_int;
    if ( i == in_range[index] ) {
      cout << i << ", " << index << ", " << in_range[index] << ", " << filter[index] << "\n";
      if ( filter[index] == 1 ) {
	coordinate[index2] = temp_int;
	index2++;
      }
      index++;
    }
  }
  //cout << "index = " << index << "\n";
  NumVariants = index2;
  input.close( );
  for ( int i = 0 ; i < index2 ; i++ ) {
    //cout << "coordinate" << i << " = " << coordinate[i] << "\n";
  }

  /****************Initialize random number generator*********/
  gsl_rng_env_setup( );
  T = gsl_rng_default;
  rgen = gsl_rng_alloc( T );

  /************ Set window sizes **************/
  int region_size = coordinate[NumVariants-1] - coordinate[0] + 1;
  int max_win_size = region_size / 2 - 1;
  NumSizes = 0;
  for ( int k = 0 ; k < num ; k++ ) {
    if ( sizes[k] <= max_win_size ) {
      NumSizes++;
    }
  }
  if ( NumSizes == 0 ) {
    cout << "Region size insufficient\n";
    exit(0);
  }

  windowSizes = new int[NumSizes];
  NumWindows = new int[NumSizes];
  winStart = new int*[NumSizes];
  winEnd = new int*[NumSizes];
  for ( int i = 0 ; i < NumSizes ; i++ ) {
    windowSizes[i] = sizes[i];
    winStart[i] = new int[2*NumVariants];
    winEnd[i] = new int[2*NumVariants];
  }

  /*************** Set remaining arrays ******************/
  N = new int[NumVariants];
  Y = new int[NumVariants];
  C = new int[NumVariants];
  Yu = new int[NumVariants];
  Nu = new int[NumVariants];
  Map = new int[NumVariants];
  NcY = new double[NumVariants];
  MAF = new double[NumVariants];
  concat = new int[NumVariants];
  backup = new int[NumVariants];

  /************* Calls to private methods ****************/
  reduce_data( );
  calc_windows( );

}

/*************** Destructor ******************************/
scanData::~scanData( ) {

  /************* Delete arrays ***********************/
  for ( int i = 0 ; i < NumSubjects ; i++ ) {
    delete[] genotype[i];
  }
  for ( int i = 0 ; i < NumSizes ; i++ ) {
    delete[] winStart[i];
    delete[] winEnd[i];
  }
  delete[] genotype;
  delete[] phenotype;
  delete[] coordinate;
  delete[] N;
  delete[] Y;
  delete[] C;
  delete[] Nu;
  delete[] Yu;
  delete[] Map;
  delete[] MAF;
  delete[] windowSizes;
  delete[] NumWindows;

  /*********** Free GSL random number generator **************/
  gsl_rng_free( rgen );

}

/*************** Data Manipulation ***********************/
void scanData::reduce_data( ) {

  cout << "Reduce Data\n";
  /************** Calculate N, Y, and MAF ****************/

  // MAF
  NumVarRet = 0;
  int Ncount;
  for ( int i = 0 ; i < NumVariants ; i++ ) {
    Ncount = 0;
    MAF[i] = 0;
    for ( int j = 0 ; j < NumSubjects ; j++ ) {
      if ( phenotype[j] == 0 ) {
	MAF[i] += genotype[j][i];
      }
      Ncount += genotype[j][i];
    }
    MAF[i] /= (2.0 * NumControl);
    if ( (MAF[i] <= MAX_MAF) & (Ncount > 0) ) {
      NumVarRet++;
    }
  }

  // N, Y
  int N1, Y1;
  int pos = 0;
  //cout << "\tSubMark3\n";
  cout << "MAX_MAF = " << MAX_MAF << "\n";
  for ( int i = 0 ; i < NumVariants ; i++ ) {
    if ( MAF[i] <= MAX_MAF ) {
      N1 = 0;
      Y1 = 0;
      for ( int j = 0 ; j < NumSubjects ; j++ ) {
	N1 += genotype[j][i];
	if ( phenotype[j] == 1 ) {
	  Y1 += genotype[j][i];
	}
      }
      if ( N1 > 0 ) {
	N[pos] = N1;
	Y[pos] = Y1;
	C[pos] = coordinate[i];
	pos++;
      }
    }
  }

  /************** Find unique pairs ****************/
  int Ymax = Y[0];
  for ( int i = 1 ; i < NumVarRet ; i++ ) {
    if ( Y[i] > Ymax ) {
      Ymax = Y[i];
    }
  }

  double ly_y = log10(Ymax);
  int digits_y = floor(ly_y) + 1;

  // If variants exist in cases
  if ( Ymax > 0 ) {
    for ( int i = 0 ; i < NumVarRet ; i++ ) {
      concat[i] = (N[i] * pow((double)10,digits_y)) + Y[i];
      backup[i] = concat[i];
    }
    sort(concat,concat+NumVarRet);
    int * lastUnique = unique(concat,concat+NumVarRet);
    NumUnique = distance(concat,lastUnique);
    double reduce;
    for ( int i = 0 ; i < NumUnique ; i++ ) {
      reduce = concat[i] / pow((double)10,digits_y);
      Nu[i] = floor(reduce);
      Yu[i] = concat[i] - (Nu[i] * pow((double)10,digits_y));
    }
  } else { // If no variants exist in cases
    for ( int i = 0 ; i < NumVarRet ; i++ ) {
      concat[i] = N[i];
      backup[i] = concat[i];
    }
    sort(concat,concat+NumVarRet);
    int * lastUnique = unique(concat,concat+NumVarRet);
    NumUnique = distance(concat,lastUnique);
    double reduce;
    for ( int i = 0 ; i < NumUnique ; i++ ) {
      reduce = concat[i];
      Nu[i] = floor(reduce);
      Yu[i] = 0;
    }
  }

  /********** Calculate Map ***************/
  for ( int i = 0 ; i < NumVarRet ; i++ ) {
    Map[i] = 0;
    for ( int j = 1 ; j < NumUnique ; j++ ) {
      if ( backup[i] == concat[j] ) {
	Map[i] = j;
      }
    }
  }

  /*********** Calculate NcY ****************/
  for ( int i = 0 ; i < NumUnique ; i++ ) {
    NcY[i] = gsl_sf_lnchoose( Nu[i], Yu[i] );
  }

}

void scanData::calc_windows( ) {
  cout << "Start Windows\n";
  /*********** Find boundaries **********/
  cout << NumVarRet << "\n";
  int first, last, ind_first, ind_last;
  int numInWindow;
  int maxInWindow = 0;
  int minInWindow = NumVarRet;
  int winCount;
  int upper, lower;
  int begin = C[0];
  int end = C[NumVarRet-1];
  int found, begin2;
  for ( int i = 0 ; i < NumSizes ; i++ ) {
    cout << "Wsize = " << i << "\n";
    winCount = 1;
    lower = begin;
    upper = begin + windowSizes[i] - 1;

    // First window
    first = end;
    last = begin;
    found = 0; // Has the first non-empty window been found?
    begin2 = begin;
    while ( found == 0 ) {
      for ( int k = 0 ; k < NumVarRet ; k++ ) {
	if ( (C[k] >= lower) & (C[k] <= upper) ) {
	  found = 1;
	  if ( C[k] <= first ) {
	    first = C[k];
	    ind_first = k;
	  }
	  if ( C[k] >= last ) {
	    last = C[k];
	    ind_last = k;
	  }
	}
      }
      if ( !found ) {
	lower++;
	upper++;
	begin2++;
      }

      winStart[i][0] = ind_first;
      winEnd[i][0] = ind_last;
      numInWindow = ind_last - ind_first + 1;
      //cout << "numInWindow = " << numInWindow << "\n";
      //cout << ind_last << "\t" << ind_first << "\n";
      if ( numInWindow < minInWindow ) {
	minInWindow = numInWindow;
      }
      if ( numInWindow > maxInWindow ) {
	maxInWindow = numInWindow;
      }
      //cout << C[winStart[i][0]] - C[winEnd[i][0]] << "\n";
    }

    // Succesive windows
    for ( int j = (begin2+1) ; j < (end-windowSizes[i]+1) ; j++ ) {
      lower++;
      upper++;
      first = end;
      last = begin;
      for ( int k = 0 ; k < NumVarRet ; k++ ) {
	if ( (C[k] >= lower) & (C[k] <= upper) ) {
	  if ( C[k] < first ) {
	    first = C[k];
	    ind_first = k;
	  }
	  if ( C[k] > last ) {
	    last = C[k];
	    ind_last = k;
	  }
	}
      }

      if ( (ind_first > winStart[i][winCount-1]) | (ind_last > winEnd[i][winCount-1]) ) {
	if ( !((first==end)&(last==begin)) ) {
	  winStart[i][winCount] = ind_first;
	  winEnd[i][winCount] = ind_last;
	  numInWindow = ind_last - ind_first + 1;
	  //cout << "numInWindow = " << numInWindow << "\n";
	  //cout << ind_last << "\t" << ind_first << "\n";
	  if ( numInWindow < minInWindow ) {
	    minInWindow = numInWindow;
	  }
	  if ( numInWindow > maxInWindow ) {
	    maxInWindow = numInWindow;
	  }
	  //cout << C[winStart[i][winCount]] - C[winEnd[i][winCount]] << "\n";
	  winCount++;
	}
      }
    }
    NumWindows[i] = winCount;
    cout << winCount << "\t" << i << "\n";
    cout << "maxInWindow = " << maxInWindow << "\n";
    cout << "minInWindow = " << minInWindow << "\n";

  }
  //exit(0);
  cout << "End Windows\n";
}

void scanData::permutation_test_rows( ) {

  /********* Permute and reset data ***********/
  gsl_ran_shuffle( rgen, phenotype, NumSubjects, sizeof(int) );
  reduce_data( );
  calc_windows( );
  //cout << "NumVariants = " << NumVariants << "\n";
  //cout << "NumVarRet = " << NumVarRet << "\n";

}

void scanData::permutation_test_columns( ) {

  gsl_ran_shuffle( rgen, Map, NumVarRet, sizeof(int) );

}



/*************** Data Access **************************/

int scanData::yu( int i ) {
  return Yu[i];
}

int scanData::nu( int i ) {
  return Nu[i];
}

int scanData::map( int i ) {
  return Map[i];
}

int scanData::c( int i ) {
  return C[i];
}

double scanData::maf( int i ) {
  return MAF[i];
}

double scanData::ncy( int i ) {
  return NcY[i];
}

int scanData::numVariants( ) {
  return NumVariants;
}

int scanData::numVarRet( ) {
  return NumVarRet;
}

int scanData::numUnique( ) {
  return NumUnique;
}

int scanData::numSizes( ) {
  return NumSizes;
}

int scanData::WindowSizes( int i ) {
  return windowSizes[i];
}

int scanData::WinStart( int j, int i ) {
  return winStart[j][i];
}

int scanData::WinEnd( int j, int i ) {
  return winEnd[j][i];
}

int scanData::numWindows( int i ) {
  return NumWindows[i];
}
