
#ifndef __SCANMODEL_EB_MCC_HH__
#define __SCANMODEL_EB_MCC_HH__

#include "ScanData.hh"
#include <gsl/gsl_multimin.h>
#include <limits>

class scanModel {

protected:

  /************ Model parameters *******************/
  double * Alpha;
  double * Beta;
  double * Pi;

  double * Alpha_m1;
  double * Beta_m1;

  double * Alpha_max;
  double * Beta_max;
  double * Alpha_m1_max;
  double * Beta_m1_max;
  double * Pi_max;

  /********* Functions of model parameters **********/
  double ln_beta_func_0;
  double ln_beta_func_1;
  double * ln_beta_post_0;
  double * ln_beta_post_1;
  double * ln_pi;
  double * ln_1_minus_pi;

  double ln_beta_func_0_m1;
  double ln_beta_func_1_m1;
  double * ln_beta_post_0_m1;
  double * ln_beta_post_1_m1;


  /********* EM algorithm variables *****************/
  double * psi;

  /********* Likelihood values **************/
  double LL_m1h0_v0;
  double LL_m1h0;
  double LL_m2h0;
  double * LL_max_m1h1;
  double * LL_max_m1h1_v0;
  double * LL_max_m2h1;
  int * LL_max_ind1_m1;
  int * LL_max_ind1_m1_v0;
  int * LL_max_ind1_m2;
  int * LL_max_ind2_m1;
  int * LL_max_ind2_m1_v0;
  int * LL_max_ind2_m2;
  double ** bayesFactor_m1;
  double ** bayesFactor_m1_v0;
  double ** bayesFactor_m2;
  double * posterior;
  int * converged;
  int * converged_m1;
  int * converged_perm;
  int converge_flag;

  /******** scanData class object **********/
  scanData * sd;

  /********** Window boundaries ************/
  int win_u;
  int win_l;

  /********* GSL Minimizer ****************/
  const gsl_multimin_fminimizer_type * T;
  gsl_multimin_fminimizer * s;
  gsl_vector * step, * minimizer_x;
  gsl_multimin_function func;

  /******* Protected methods ****************/
  static double calculate_Q ( const gsl_vector *, void * );
  static double model_1_objective ( const gsl_vector *, void * );
  void estimate_pi ( int, int );
  void expectation ( int, int );
  void estimate_ab ( );
  void estimate_ab_m1 ( );
  void EM_algorithm ( );
  void EM_algorithm_pi ( int, int );
  double calculate_LL_model_1 ( int, int );
  double calculate_LL_model_1_v0 ( int, int );
  double calculate_LL_model_2 ( int, int );

public:
  scanModel( scanData *, double *, double *, double *);
  void set( double *, double *, double * );

  // Run scan statistic
  void run_scan( int );

  /**************** Accessor methods **********/
  double get_alpha( int );
  double get_beta( int );
  double get_alpha_m1( int );
  double get_beta_m1( int );
  double get_pi( int );
  double get_bayesFactor_m1( int j, int i ){ return bayesFactor_m1[j][i]; }
  double get_bayesFactor_m1_v0( int j, int i ){ return bayesFactor_m1_v0[j][i]; }
  double get_bayesFactor_m2( int j, int i ){ return bayesFactor_m2[j][i]; }
  double get_posterior( int i ){ return posterior[i]; }
  double get_LL_max_m1h1( int j ) { return LL_max_m1h1[j]; }
  double get_LL_max_m1h1_v0( int j ) { return LL_max_m1h1_v0[j]; }
  double get_LL_max_m2h1( int j ) { return LL_max_m2h1[j]; }
  double get_LL_max_m1h0( ) { return LL_m1h0; }
  double get_LL_max_m1h0_v0( ) { return LL_m1h0_v0; }
  double get_LL_max_m2h0( ) { return LL_m2h0; }
  int get_win_start_m1( );
  int get_win_size_m1( int j ) { return LL_max_ind1_m1[j]; }
  int get_win_locus_m1( int i ) { return LL_max_ind2_m1[i]; }
  int get_win_size_m1_v0( int j ) { return LL_max_ind1_m1_v0[j]; }
  int get_win_locus_m1_v0( int i ) { return LL_max_ind2_m1_v0[i]; }
  int get_win_size_m2( int j ) { return LL_max_ind1_m2[j]; }
  int get_win_locus_m2( int i ) { return LL_max_ind2_m2[i]; }
  int get_converged( int i ) { return converged[i]; }
  int get_converged_m1( int i ) { return converged_m1[i]; }

  void set( int, int, double *, double *, double * );
};

#endif
