
#include "ScanData.hh"
#include "ScanModel.hh"
#include "LLtest.hh"

using namespace std;

main ( int argc, const char ** argv ) {

  // Read in command line arguments
  const char * ped;
  const char * var;
  const char * inc;
  const char * save;
  const char * save2;
  string win_sizes;
  string range;
  int permute_type = 0;
  int num_perm;
  double maf_max;
  if ( argc == 10 ) {
    ped = argv[1];
    var = argv[2];
    inc = argv[3];
    save = argv[4];
    save2 = argv[5];
    win_sizes = argv[6];
    maf_max = atof(argv[7]);
    range = argv[8];
    num_perm = atoi(argv[9]);
  } else {
    cout << "Usage: eBScan <ped_file> <var_file> <include> <save_model> <save_path> <win_sizes> <max_maf> <range> <num_perm>\n";
    exit(0);
  }

  // Open file to write results too
  ofstream save_max;
  ofstream save_path;
  save_max.open( save );
  save_path.open( save2 );

  // Construct scanData object
  // Window sizes: Add command line option later
  string parsed;
  stringstream iss(win_sizes);
  int num = 0;
  while ( getline(iss,parsed,',') ) {
    num++;
  }
  iss.clear();
  iss.str("");
  iss << win_sizes;
  int * sizes = new int[num];
  for ( int i = 0 ; i < num ; i++ ) {
    getline(iss,parsed,',');
    sizes[i] = atoi(parsed.c_str());
  }
  iss.clear();
  iss.str("");
  iss << range;
  getline(iss,parsed,':');
  int chr = atoi(parsed.c_str());
  getline(iss,parsed,':');
  int l_bound = atoi(parsed.c_str());
  getline(iss,parsed,':');
  int u_bound = atoi(parsed.c_str());
  cout << "Mark1\n";
  scanData sd( ped, var, inc, sizes, num, maf_max, chr, l_bound, u_bound );
  cout << "Mark2\n";

  // Construct LRtest object
  LRtest lrtest( &sd );
  
  // Construct scanModel objects
  double alpha[2] = {4,2};
  double beta[2] = {3,3};
  double pi[2] = {.1,.5};
  scanModel scan1( &sd, alpha, beta, pi );

  // Running models
  scan1.run_scan( 0 );
  lrtest.calc_LLR_all( );
  cout << "Finished\n";

  save_max << "\tAlpha1\tAlpha2\tBeta1\tBeta2\n";
  save_max << "EBSCAN1\t" << scan1.get_alpha_m1(0) << "\t" << scan1.get_alpha_m1(1) << "\t" << scan1.get_beta_m1(0) << "\t" << scan1.get_beta_m1(1) << "\n";
  save_max << "EBSCAN2\t" << scan1.get_alpha(0) << "\t" << scan1.get_alpha(1) << "\t" << scan1.get_beta(0) << "\t" << scan1.get_beta(1) << "\n*******\n";
  save_path << "Start\tEnd\tEBSCAN1\tEBSCAN2\tLRT\n";
  for ( int i = 0 ; i < (sd.numSizes()) ; i++ ) {
    //cout << i << "\n";
    //save_max << scan1.get_LL_max_m1h1_v0( i ) << "\t" << scan1.get_LL_max_m1h0_v0( ) << "\t";
    //save_max << scan1.get_LL_max_m2h1( i ) << "\t" << scan1.get_LL_max_m2h0( ) << ";\t";
    for ( int k = 0 ; k < sd.numWindows(i) ; k++ ) {
      save_path << sd.c(sd.WinStart(i,k)) << "\t" << sd.c(sd.WinEnd(i,k)) << "\t" << scan1.get_bayesFactor_m1_v0(i,k) << "\t" << scan1.get_bayesFactor_m2(i,k) << "\t" << lrtest.get_LLR(i,k)  << "\n";
    }
    save_path << "\n";
  }
  cout << "NumVarRet = " << sd.numVarRet() << "\n";
  save_path << "Coordinate\tCases\tTotal\tPosterior\n";
  for ( int k = 0 ; k < sd.numVarRet() ; k++ ) {
    save_path << sd.c(k) << "\t" << sd.yu(sd.map(k)) << "\t" << sd.nu(sd.map(k)) << "\t" << scan1.get_posterior(k) << "\n";
  }
  save_max << "\n*******\n";
  save_path.flush();
  save_max.flush();

  // Reporting clusters
  int m1h1_lower, m1h1_upper, m1h1_lower_v0, m1h1_upper_v0, m2h1_lower, m2h1_upper, lrtest_lower, lrtest_upper;
  int skat_lower, skat_upper, burden_lower, burden_upper;
  save_max << "EBSCAN1\tEBSCAN2\tLRT\n";
  for ( int i = 0 ; i < (sd.numSizes( )+1) ; i++ ) {
    m1h1_lower_v0 = sd.c(scan1.get_win_size_m1_v0( i ));
    m1h1_upper_v0 = sd.c(scan1.get_win_locus_m1_v0( i ));
    m2h1_lower = sd.c(scan1.get_win_size_m2( i ));
    m2h1_upper = sd.c(scan1.get_win_locus_m2( i ));
    lrtest_lower = sd.c(lrtest.get_LLRmax_size( i ));
    lrtest_upper = sd.c(lrtest.get_LLRmax_window( i ));
    save_max << "(" << m1h1_lower_v0 << "," << m1h1_upper_v0 << ")\t(" << m2h1_lower << "," << m2h1_upper << ")\t(" << lrtest_lower << "," << lrtest_upper << ")\n";
    save_max.flush();
  }
  save_max << "\n";
  save_max.flush();

  int loops = num_perm;
  int min_count = num_perm / 20;
  int stop = 0;
  double *bayesFactor1, *bayesFactor1v0, *bayesFactor2, *bayesFactor1_perm, *bayesFactor1v0_perm, *bayesFactor2_perm, *LLR, *LLR_perm;
  bayesFactor1 = new double[sd.numSizes()+1];
  bayesFactor1v0 = new double[sd.numSizes()+1];
  bayesFactor2 = new double[sd.numSizes()+1];
  bayesFactor1_perm = new double[sd.numSizes()+1];
  bayesFactor1v0_perm = new double[sd.numSizes()+1];
  bayesFactor2_perm = new double[sd.numSizes()+1];
  LLR = new double[sd.numSizes()+1];
  LLR_perm = new double[sd.numSizes()+1];
  for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
    bayesFactor1v0[i] = scan1.get_LL_max_m1h1_v0( i ) - scan1.get_LL_max_m1h0_v0( );
    bayesFactor2[i] = scan1.get_LL_max_m2h1( i ) - scan1.get_LL_max_m2h0( );
    LLR[i] = lrtest.get_LLRmax(i);
  }
  double *counts1, *counts1v0, *counts2, *countsLLR;
  counts1v0 = new double[sd.numSizes()+1];
  counts2 = new double[sd.numSizes()+1];
  countsLLR = new double[sd.numSizes()+1];
  for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
    counts1v0[i] = 0;
    counts2[i] = 0;
    countsLLR[i] = 0;
  }

  cout << "Starting loop\n";
  cout << "NumVarRet = " << sd.numVarRet() << "\n";

  for ( int k = 0 ; k < (sd.numSizes()+1) ; k++ ) {
    save_max << "EBSCAN1\tEBSCAN2\tLRT\t";
  }
  save_max << "\n";

  for ( int indL = 0 ; indL < loops ; indL++ ) {
    cout << "Loop " << indL << "\n";
    if ( permute_type == 1 ) {
      sd.permutation_test_columns( );
      scan1.set( alpha, beta, pi );
      scan1.run_scan( 0 );
      lrtest.calc_LLR_all( );
    } else {
      sd.permutation_test_rows( );
      scan1.set( alpha, beta, pi );
      scan1.run_scan( 0 );
      lrtest.calc_LLR_all( );
    }
    cout << "NumVarRet = " << sd.numVarRet() << "\n";

    for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
      bayesFactor1v0_perm[i] = scan1.get_LL_max_m1h1_v0( i ) - scan1.get_LL_max_m1h0_v0( );
      bayesFactor2_perm[i] = scan1.get_LL_max_m2h1( i ) - scan1.get_LL_max_m2h0( );
      LLR_perm[i] = lrtest.get_LLRmax(i);
    }

    for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
      if ( bayesFactor1v0_perm[i] >= bayesFactor1v0[i] ) {
	counts1v0[i]++;
      }
      if ( bayesFactor2_perm[i] >= bayesFactor2[i] ) {
	counts2[i]++;
      }
      if ( LLR_perm[i] >= LLR[i] ) {
	countsLLR[i]++;
      }
    }

    if ( ((indL+1) % 10) == 0 ) {
      for ( int k = 0 ; k < (sd.numSizes()+1) ; k++ ) {
	save_max << counts1v0[k] << "\t" << counts2[k] << "\t" << countsLLR[k] << "\t";
      }
      save_max << "\n";
      save_max.flush();
    }

    stop = 1;
    if ( counts1v0[sd.numSizes()] < min_count ) {
      stop = 0;
    }
    if ( counts2[sd.numSizes()] < min_count ) {
      stop = 0;
    }
    if ( countsLLR[sd.numSizes()] < min_count ) {
      stop = 0;
    }
    if ( stop == 1 ) {
      for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
	counts1v0[i] /= loops;
	counts2[i] /= loops;
	countsLLR[i] /= loops;
	save_max << counts1v0[i] << "\t" << counts2[i] << "\t" << countsLLR[i] << "\t";
      }
      save_max << "\tstopped" << "\n";
      save_max.flush();
      exit(0);
    }

  }
  for ( int i = 0 ; i < (sd.numSizes()+1) ; i++ ) {
    counts1v0[i] /= loops;
    counts2[i] /= loops;
    countsLLR[i] /= loops;
    save_max << counts1v0[i] << "\t" << counts2[i] << "\t" << countsLLR[i] << "\t";
  }
  save_max << "\n";
  save_max.flush();

}
